import json
import os
import tempfile

from detect_secrets import SecretsCollection
from detect_secrets.settings import default_settings

from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.ssm.ssm_client import ssm_client


class ssm_document_secrets(Check):
    def execute(self):
        findings = []
        for document in ssm_client.documents.values():
            report = Check_Report_AWS(self.metadata())
            report.region = document.region
            report.resource_arn = document.arn
            report.resource_id = document.name
            report.resource_tags = document.tags
            report.status = "PASS"
            report.status_extended = (
                f"No secrets found in SSM Document {document.name}."
            )

            if document.content:
                temp_env_data_file = tempfile.NamedTemporaryFile(delete=False)
                temp_env_data_file.write(
                    bytes(
                        json.dumps(document.content, indent=2),
                        encoding="raw_unicode_escape",
                    )
                )
                temp_env_data_file.close()
                secrets = SecretsCollection()
                with default_settings():
                    secrets.scan_file(temp_env_data_file.name)

                detect_secrets_output = secrets.json()
                if detect_secrets_output:
                    secrets_string = ", ".join(
                        [
                            f"{secret['type']} on line {secret['line_number']}"
                            for secret in detect_secrets_output[temp_env_data_file.name]
                        ]
                    )
                    report.status = "FAIL"
                    report.status_extended = f"Potential secret found in SSM Document {document.name} -> {secrets_string}."

                os.remove(temp_env_data_file.name)

            findings.append(report)

        return findings



from typing import Optional

from pydantic import BaseModel

from prowler.lib.logger import logger
from prowler.lib.scan_filters.scan_filters import is_resource_filtered
from prowler.providers.aws.lib.service.service import AWSService


################## Athena
class Athena(AWSService):
    def __init__(self, audit_info):
        # Call AWSService's __init__
        super().__init__(__class__.__name__, audit_info)
        self.workgroups = {}
        self.__threading_call__(self.__list_workgroups__)
        self.__get_workgroups__()
        self.__list_tags_for_resource__()

    def __list_workgroups__(self, regional_client):
        logger.info("Athena - Listing WorkGroups...")
        try:
            list_workgroups = regional_client.list_work_groups()
            for workgroup in list_workgroups["WorkGroups"]:
                workgroup_name = workgroup["Name"]
                workgroup_arn = f"arn:{self.audited_partition}:athena:{regional_client.region}:{self.audited_account}:workgroup/{workgroup_name}"
                if not self.audit_resources or (
                    is_resource_filtered(workgroup_arn, self.audit_resources)
                ):
                    self.workgroups[workgroup_arn] = WorkGroup(
                        arn=workgroup_arn,
                        name=workgroup_name,
                        region=regional_client.region,
                    )

        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __get_workgroups__(self):
        logger.info("Athena - Getting WorkGroups...")
        try:
            for workgroup in self.workgroups.values():
                wg = self.regional_clients[workgroup.region].get_work_group(
                    WorkGroup=workgroup.name
                )

                wg_configuration = wg.get("WorkGroup").get("Configuration")
                self.workgroups[
                    workgroup.arn
                ].enforce_workgroup_configuration = wg_configuration.get(
                    "EnforceWorkGroupConfiguration", False
                )

                # We include an empty EncryptionConfiguration to handle if the workgroup does not have encryption configured
                encryption = (
                    wg_configuration.get(
                        "ResultConfiguration",
                        {"EncryptionConfiguration": {}},
                    )
                    .get(
                        "EncryptionConfiguration",
                        {"EncryptionOption": ""},
                    )
                    .get("EncryptionOption")
                )

                if encryption in ["SSE_S3", "SSE_KMS", "CSE_KMS"]:
                    encryption_configuration = EncryptionConfiguration(
                        encryption_option=encryption, encrypted=True
                    )
                    self.workgroups[
                        workgroup.arn
                    ].encryption_configuration = encryption_configuration

        except Exception as error:
            logger.error(
                f"{self.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __list_tags_for_resource__(self):
        logger.info("Athena - Listing Tags...")
        try:
            for workgroup in self.workgroups.values():
                regional_client = self.regional_clients[workgroup.region]
                workgroup.tags = regional_client.list_tags_for_resource(
                    ResourceARN=workgroup.arn
                )["Tags"]
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )


class EncryptionConfiguration(BaseModel):
    encryption_option: str
    encrypted: bool


class WorkGroup(BaseModel):
    arn: str
    name: str
    encryption_configuration: EncryptionConfiguration = EncryptionConfiguration(
        encryption_option="", encrypted=False
    )
    enforce_workgroup_configuration: bool = False
    region: str
    tags: Optional[list] = []



import sys

from pydantic import parse_obj_as

from prowler.lib.check.compliance_models import Compliance_Base_Model
from prowler.lib.check.models import Check_Metadata_Model
from prowler.lib.logger import logger


def update_checks_metadata_with_compliance(
    bulk_compliance_frameworks: dict, bulk_checks_metadata: dict
):
    """Update the check metadata model with the compliance framework"""
    try:
        for check in bulk_checks_metadata:
            check_compliance = []
            for framework in bulk_compliance_frameworks.values():
                for requirement in framework.Requirements:
                    compliance_requirements = []
                    # Verify if check is in the requirement
                    if check in requirement.Checks:
                        # Include the requirement into the check's framework requirements
                        compliance_requirements.append(requirement)
                        # Create the Compliance_Model
                        compliance = Compliance_Base_Model(
                            Framework=framework.Framework,
                            Provider=framework.Provider,
                            Version=framework.Version,
                            Description=framework.Description,
                            Requirements=compliance_requirements,
                        )
                        # Include the compliance framework for the check
                        check_compliance.append(compliance)
            # Save it into the check's metadata
            bulk_checks_metadata[check].Compliance = check_compliance

        # Add requirements of Manual Controls
        for framework in bulk_compliance_frameworks.values():
            for requirement in framework.Requirements:
                compliance_requirements = []
                # Verify if requirement is Manual
                if not requirement.Checks:
                    compliance_requirements.append(requirement)
                    # Create the Compliance_Model
                    compliance = Compliance_Base_Model(
                        Framework=framework.Framework,
                        Provider=framework.Provider,
                        Version=framework.Version,
                        Description=framework.Description,
                        Requirements=compliance_requirements,
                    )
                    # Include the compliance framework for the check
                    check_compliance.append(compliance)
            # Create metadata for Manual Control
            manual_check_metadata = {
                "Provider": "aws",
                "CheckID": "manual_check",
                "CheckTitle": "Manual Check",
                "CheckType": [],
                "ServiceName": "",
                "SubServiceName": "",
                "ResourceIdTemplate": "",
                "Severity": "",
                "ResourceType": "",
                "Description": "",
                "Risk": "",
                "RelatedUrl": "",
                "Remediation": {
                    "Code": {"CLI": "", "NativeIaC": "", "Other": "", "Terraform": ""},
                    "Recommendation": {"Text": "", "Url": ""},
                },
                "Categories": [],
                "Tags": {},
                "DependsOn": [],
                "RelatedTo": [],
                "Notes": "",
            }
            manual_check = parse_obj_as(Check_Metadata_Model, manual_check_metadata)
            # Save it into the check's metadata
            bulk_checks_metadata["manual_check"] = manual_check
            bulk_checks_metadata["manual_check"].Compliance = check_compliance

        return bulk_checks_metadata
    except Exception as e:
        logger.critical(f"{e.__class__.__name__}[{e.__traceback__.tb_lineno}] -- {e}")
        sys.exit(1)
